@@ -46,7 +46,7 @@ from py7zr.callbacks import ExtractCallback
 from py7zr.compressor import SupportedMethods, get_methods_names
 from py7zr.exceptions import Bad7zFile, CrcError, DecompressionError, InternalError, UnsupportedCompressionMethodError
 from py7zr.helpers import ArchiveTimestamp, MemIO, NullIO, calculate_crc32, filetime_to_dt, readlink
-from py7zr.properties import DEFAULT_FILTERS, MAGIC_7Z, get_default_blocksize
+from py7zr.properties import DEFAULT_FILTERS, MAGIC_7Z, get_default_blocksize, FILTER_DEFLATE64
 from py7zr.win32compat import is_windows_native_python, is_windows_unc_path
 
 if sys.platform.startswith("win"):
@@ -132,17 +132,23 @@ class ArchiveFile:
     @property
     def archivable(self) -> bool:
         """File has a Windows `archive` flag."""
-        return self._test_attribute(stat.FILE_ATTRIBUTE_ARCHIVE)  # noqa
+        if hasattr(stat, "FILE_ATTRIBUTE_ARCHIVE"):
+            return self._test_attribute(getattr(stat, "FILE_ATTRIBUTE_ARCHIVE"))
+        return False
 
     @property
     def is_directory(self) -> bool:
         """True if file is a directory, otherwise False."""
-        return self._test_attribute(stat.FILE_ATTRIBUTE_DIRECTORY)  # noqa
+        if hasattr(stat, "FILE_ATTRIBUTE_DIRECTORY"):
+            return self._test_attribute(getattr(stat, "FILE_ATTRIBUTE_DIRECTORY"))
+        return False
 
     @property
     def readonly(self) -> bool:
         """True if file is readonly, otherwise False."""
-        return self._test_attribute(stat.FILE_ATTRIBUTE_READONLY)  # noqa
+        if hasattr(stat, "FILE_ATTRIBUTE_READONLY"):
+            return self._test_attribute(getattr(stat, "FILE_ATTRIBUTE_READONLY"))
+        return False
 
     def _get_unix_extension(self) -> Optional[int]:
         attributes = self._get_property("attributes")
@@ -163,12 +169,18 @@ class ArchiveFile:
         e = self._get_unix_extension()
         if e is not None:
             return stat.S_ISLNK(e)
-        return self._test_attribute(stat.FILE_ATTRIBUTE_REPARSE_POINT)  # noqa
+        if hasattr(stat, "FILE_ATTRIBUTE_REPARSE_POINT"):
+            return self._test_attribute(getattr(stat, "FILE_ATTRIBUTE_REPARSE_POINT"))
+        return False
 
     @property
     def is_junction(self) -> bool:
         """True if file is a junction/reparse point on windows, otherwise False."""
-        return self._test_attribute(stat.FILE_ATTRIBUTE_REPARSE_POINT | stat.FILE_ATTRIBUTE_DIRECTORY)  # noqa  # noqa
+        if hasattr(stat, "FILE_ATTRIBUTE_REPARSE_POINT"):
+            return self._test_attribute(
+                getattr(stat, "FILE_ATTRIBUTE_REPARSE_POINT") | getattr(stat, "FILE_ATTRIBUTE_DIRECTORY")
+            )
+        return False
 
     @property
     def is_socket(self) -> bool:
@@ -533,10 +545,6 @@ class SevenZipFile(contextlib.AbstractContextManager):
         fnames: List[str] = []  # check duplicated filename in one archive?
         self.q.put(("pre", None, None))
         for f in self.files:
-            # TODO: sanity check
-            # check whether f.filename with invalid characters: '../'
-            if f.filename.startswith("../"):
-                raise Bad7zFile
             # When archive has a multiple files which have same name
             # To guarantee order of archive, multi-thread decompression becomes off.
             # Currently always overwrite by latter archives.
@@ -551,10 +559,23 @@ class SevenZipFile(contextlib.AbstractContextManager):
                         break
                     i += 1
             fnames.append(outname)
-            if path is not None:
-                outfilename = path.joinpath(outname)
+            # check f.filename has invalid directory traversals
+            if path is None:
+                # do following but is_relative_to introduced in py 3.9
+                # so I replaced it with relative_to. when condition is not satisfied, raise ValueError
+                # if not pathlib.Path(...).joinpath(outname.lstrip("./")).is_relative_to(...):
+                #    raise Bad7zFile
+                try:
+                    pathlib.Path(os.getcwd()).joinpath(outname.lstrip("./")).relative_to(os.getcwd())
+                except ValueError:
+                    raise Bad7zFile
+                outfilename = pathlib.Path(outname.lstrip("./"))
             else:
-                outfilename = pathlib.Path(outname)
+                outfilename = path.joinpath(outname.lstrip("./"))
+                try:
+                    outfilename.relative_to(path)
+                except ValueError:
+                    raise Bad7zFile
             # When python on Windows and not python on Cygwin,
             # Add win32 file namespace to exceed microsoft windows
             # path length limitation to 260 bytes
@@ -645,7 +666,9 @@ class SevenZipFile(contextlib.AbstractContextManager):
         elif filters is None:
             filters = DEFAULT_FILTERS.ARCHIVE_FILTER
         else:
-            pass
+            for f in filters:
+                if f["id"] == FILTER_DEFLATE64:
+                    raise UnsupportedCompressionMethodError(filters, "Compression with deflate64 is not supported.")
         self.header.filters = filters
         self.header.password = password
         if self.header.main_streams is not None:
@@ -661,7 +684,9 @@ class SevenZipFile(contextlib.AbstractContextManager):
         elif filters is None:
             filters = DEFAULT_FILTERS.ARCHIVE_FILTER
         else:
-            pass
+            for f in filters:
+                if f["id"] == FILTER_DEFLATE64:
+                    raise UnsupportedCompressionMethodError(filters, "Compression with deflate64 is not supported.")
         self.files = ArchiveFileList()
         self.sig_header = SignatureHeader()
         self.sig_header._write_skelton(self.fp)
@@ -757,7 +782,7 @@ class SevenZipFile(contextlib.AbstractContextManager):
             self.header_encryption = False
 
     @staticmethod
-    def _check_7zfile(fp: Union[BinaryIO, io.BufferedReader]) -> bool:
+    def _check_7zfile(fp: Union[BinaryIO, io.BufferedReader, io.IOBase]) -> bool:
         result = MAGIC_7Z == fp.read(len(MAGIC_7Z))[: len(MAGIC_7Z)]
         fp.seek(-len(MAGIC_7Z), 1)
         return result
@@ -839,37 +864,37 @@ class SevenZipFile(contextlib.AbstractContextManager):
                     fstat = target.stat()
                     if stat.S_ISDIR(fstat.st_mode):
                         f["emptystream"] = True
-                        f["attributes"] = stat.FILE_ATTRIBUTE_DIRECTORY  # noqa
+                        f["attributes"] = getattr(stat, "FILE_ATTRIBUTE_DIRECTORY")
                         f["attributes"] |= FILE_ATTRIBUTE_UNIX_EXTENSION | (stat.S_IFDIR << 16)
                         f["attributes"] |= stat.S_IMODE(fstat.st_mode) << 16
                     else:
                         f["emptystream"] = False
-                        f["attributes"] = stat.FILE_ATTRIBUTE_ARCHIVE  # noqa
+                        f["attributes"] = getattr(stat, "FILE_ATTRIBUTE_ARCHIVE")
                         f["attributes"] |= FILE_ATTRIBUTE_UNIX_EXTENSION | (stat.S_IMODE(fstat.st_mode) << 16)
                 else:
                     f["emptystream"] = False
-                    f["attributes"] = stat.FILE_ATTRIBUTE_ARCHIVE | stat.FILE_ATTRIBUTE_REPARSE_POINT  # noqa
+                    f["attributes"] = getattr(stat, "FILE_ATTRIBUTE_ARCHIVE") | getattr(stat, "FILE_ATTRIBUTE_REPARSE_POINT")
                     f["attributes"] |= FILE_ATTRIBUTE_UNIX_EXTENSION | (stat.S_IFLNK << 16)
                     f["attributes"] |= stat.S_IMODE(fstat.st_mode) << 16
             elif target.is_dir():
                 f["emptystream"] = True
-                f["attributes"] = stat.FILE_ATTRIBUTE_DIRECTORY  # noqa
+                f["attributes"] = getattr(stat, "FILE_ATTRIBUTE_DIRECTORY")
                 f["attributes"] |= FILE_ATTRIBUTE_UNIX_EXTENSION | (stat.S_IFDIR << 16)
                 f["attributes"] |= stat.S_IMODE(fstat.st_mode) << 16
             elif target.is_file():
                 f["emptystream"] = False
                 f["uncompressed"] = fstat.st_size
-                f["attributes"] = stat.FILE_ATTRIBUTE_ARCHIVE  # noqa
+                f["attributes"] = getattr(stat, "FILE_ATTRIBUTE_ARCHIVE")
                 f["attributes"] |= FILE_ATTRIBUTE_UNIX_EXTENSION | (stat.S_IMODE(fstat.st_mode) << 16)
         else:
             fstat = target.stat()
             if target.is_dir():
                 f["emptystream"] = True
-                f["attributes"] = stat.FILE_ATTRIBUTE_DIRECTORY  # noqa
+                f["attributes"] = stat.FILE_ATTRIBUTE_DIRECTORY
             elif target.is_file():
                 f["emptystream"] = False
                 f["uncompressed"] = fstat.st_size
-                f["attributes"] = stat.FILE_ATTRIBUTE_ARCHIVE  # noqa
+                f["attributes"] = stat.FILE_ATTRIBUTE_ARCHIVE
 
         f["creationtime"] = ArchiveTimestamp.from_datetime(fstat.st_ctime)
         f["lastwritetime"] = ArchiveTimestamp.from_datetime(fstat.st_mtime)
@@ -883,7 +908,7 @@ class SevenZipFile(contextlib.AbstractContextManager):
         f["filename"] = pathlib.Path(arcname).as_posix()
         f["uncompressed"] = size
         f["emptystream"] = size == 0
-        f["attributes"] = stat.FILE_ATTRIBUTE_ARCHIVE  # noqa
+        f["attributes"] = getattr(stat, "FILE_ATTRIBUTE_ARCHIVE")
         f["creationtime"] = ArchiveTimestamp.from_now()
         f["lastwritetime"] = ArchiveTimestamp.from_now()
         return f
@@ -1116,8 +1141,10 @@ def is_7zfile(file: Union[BinaryIO, str, pathlib.Path]) -> bool:
     """
     result = False
     try:
-        if isinstance(file, io.IOBase) and hasattr(file, "read"):
-            result = SevenZipFile._check_7zfile(file)  # noqa
+        if (isinstance(file, BinaryIO) or isinstance(file, io.BufferedReader) or isinstance(file, io.IOBase)) and hasattr(
+            file, "read"
+        ):
+            result = SevenZipFile._check_7zfile(file)
         elif isinstance(file, str):
             with open(file, "rb") as fp:
                 result = SevenZipFile._check_7zfile(fp)
@@ -1194,7 +1221,9 @@ class Worker:
                             skip_notarget=skip_notarget,
                         )
                 else:
-                    filename = getattr(fp, "name", None)
+                    if getattr(fp, "name", None) is None:
+                        raise InternalError("Caught unknown variable status error")
+                    filename: str = getattr(fp, "name", "")  # do not become "" but it is for type check.
                     self.extract_single(open(filename, "rb"), empty_files, 0, 0, q)
                     concurrent_tasks = []
                     exc_q: queue.Queue = queue.Queue()
